Option Strict On
Imports System.ComponentModel
Imports ExplorerControls.ShellDll
Imports System.Runtime.InteropServices

''' <summary>
''' A ComboBox for displaying the file system
''' </summary>
''' <remarks></remarks>
<ClassInterface(ClassInterfaceType.None)> _
Public Class ExpCombo
    Inherits ComboBox

#Region " Fields And Properties "

    'For item drawing - the selected item changes continually when the DropDownList is open
    'so we need to know which folder is really selected
    Private m_selectedItem As ExpComboItem = Nothing

    Private itemsCollection As ExpComboItemCollection
    ''' <summary>
    ''' The items currently in the ExpCombo
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Overridable Shadows ReadOnly Property Items() As ExpComboItemCollection
        Get
            Return itemsCollection
        End Get
    End Property

    'Flag for Click sound
    Private m_playSound As Boolean = False

    Private m_showHiddenFolders As Boolean = True
    ''' <summary>
    ''' Determines whether hidden folders will be shown in the ExpCombo
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Behavior"), Description("Determines whether hidden folders will be shown in the ExpCombo."), _
    DefaultValue(True)> _
    Public Property ShowHiddenFolders() As Boolean
        Get
            Return m_showHiddenFolders
        End Get
        Set(ByVal Value As Boolean)
            m_showHiddenFolders = Value
        End Set
    End Property

    Private m_rootDirectory As ExpTree.StartDir = ExpTree.StartDir.None
    ''' <summary>
    ''' Root directory for the control
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Behavior"), Description("Root directory for the control."), _
    DefaultValue(GetType(ExpTree.StartDir), "None")> _
    Public Property RootDirectory() As ExpTree.StartDir
        Get
            Return m_rootDirectory
        End Get
        Set(ByVal Value As ExpTree.StartDir)
            If Array.IndexOf([Enum].GetValues(Value.GetType), Value) >= 0 Then
                'If 'None' is selected then clear everything
                If Value = ExpTree.StartDir.None Then
                    Me.Items.Clear()
                    m_startUpDirectory = Value
                    m_rootDirectory = Value
                    Exit Property
                End If

                Dim special As CShItem
                If Value = ExpTree.StartDir.Desktop Then
                    special = CShItem.GetDeskTop()
                Else
                    special = CShItem.GetCShItem(CType(Value, ShellDll.CSIDL))
                End If
                If BuildCombo(special) Then
                    'Select the root as the startup directory
                    m_rootDirectory = Value
                End If
            Else
                Throw New ApplicationException("Invalid RootDirectory")
            End If
        End Set
    End Property

    Private m_startUpDirectory As ExpTree.StartDir = ExpTree.StartDir.None
    ''' <summary>
    ''' Directory to be selected on startup
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Behavior"), Description("Directory to be selected on startup."), _
    DefaultValue(GetType(ExpTree.StartDir), "None")> _
    Public Property StartUpDirectory() As ExpTree.StartDir
        Get
            Return m_startUpDirectory
        End Get
        Set(ByVal Value As ExpTree.StartDir)
            'Exit if the combo has not been built 
            If IsNothing(Me.RootDirectory) Or (Me.RootDirectory = ExpTree.StartDir.None) Then
                Exit Property
            End If

            If Value = ExpTree.StartDir.None Then
                m_startUpDirectory = Value
                Me.SelectedIndex = -1
                Exit Property
            End If

            If Array.IndexOf([Enum].GetValues(Value.GetType), Value) >= 0 Then
                Dim special As CShItem
                If Value = ExpTree.StartDir.Desktop Then
                    special = CShItem.GetDeskTop()
                Else
                    special = CShItem.GetCShItem(CType(Value, ShellDll.CSIDL))
                End If
                If Not IsNothing(SelectFolder(special)) Then
                    m_startUpDirectory = Value
                End If
            Else
                Throw New ApplicationException("Invalid StartUpDirectory")
            End If
        End Set
    End Property

    ''' <summary>
    ''' The selected ExpComboItem
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Browsable(False), DefaultValue(CType(Nothing, Object))> _
    Public Shadows Property SelectedItem() As ExpComboItem
        Get
            Return DirectCast(MyBase.SelectedItem, ExpComboItem)
        End Get
        Set(ByVal Value As ExpComboItem)
            MyBase.SelectedItem = Value
        End Set
    End Property

    Private m_displayPath As Boolean = False
    ''' <summary>
    ''' Determines whether to show the full path in the edit area
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Behavior"), Description("Determines whether to show the full path in the edit area."), _
    DefaultValue(False)> _
    Public Property DisplayPath() As Boolean
        Get
            Return m_displayPath
        End Get
        Set(ByVal Value As Boolean)
            m_displayPath = Value
            Refresh()
        End Set
    End Property

    Private m_style As ExpComboStyle = ExpComboStyle.CurrentFolder
    ''' <summary> 
    ''' The Style of the ExpCombo
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Behavior"), Description("The Style of the ExpCombo."), _
    DefaultValue(GetType(ExpComboStyle), "CurrentFolder")> _
    Public Property Style() As ExpComboStyle
        Get
            Return m_style
        End Get
        Set(ByVal value As ExpComboStyle)
            m_style = value
            RefreshCombo()
        End Set
    End Property

    Private m_offset As Integer = 10
    ''' <summary> 
    ''' The Offset size for the folder heirachy
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Appearance"), Description("The Offset size for the folder heirachy."), _
    DefaultValue(10)> _
    Public Property Offset() As Integer
        Get
            Return m_offset
        End Get
        Set(ByVal value As Integer)
            m_offset = value
        End Set
    End Property

    'Make some base class Properties read only
    <Browsable(False), EditorBrowsable(EditorBrowsableState.Never)> _
    Public Shadows ReadOnly Property DropDownStyle() As ComboBoxStyle
        Get
            Return MyBase.DropDownStyle
        End Get
    End Property

    <Browsable(False), EditorBrowsable(EditorBrowsableState.Never)> _
    Public Shadows ReadOnly Property DrawMode() As DrawMode
        Get
            Return MyBase.DrawMode
        End Get
    End Property

#End Region

#Region " Constructor "

    ''' <summary>
    ''' Creates a new instance of the ExpCombo class
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub New()
        MyBase.New()
        itemsCollection = New ExpComboItemCollection(Me)

        'Set to OwnerDraw
        MyBase.DrawMode = Windows.Forms.DrawMode.OwnerDrawFixed
        'Must be set to DropDownList
        MyBase.DropDownStyle = ComboBoxStyle.DropDownList

    End Sub


#End Region

#Region " Drawing "

    'Draw the Items
    Protected Overrides Sub OnDrawItem(ByVal e As System.Windows.Forms.DrawItemEventArgs)

        'MyBase.OnDrawItem(e)

        'Draw the background
        Dim g As Graphics = e.Graphics
        g.FillRectangle(New SolidBrush(Me.BackColor), e.Bounds)

        If e.Index = -1 Then Exit Sub

        Dim item As ExpComboItem = Me.Items(e.Index)
        Dim index As Integer
        Dim focus As Boolean = False
        Dim open As Boolean = False
        If (e.State And DrawItemState.Focus) = DrawItemState.Focus Then
            focus = True
        End If
        If item Is m_selectedItem Then
            open = True
        End If
        index = SystemImageListManager.GetIconIndex(item.CShItem, open, focus)

        Dim ic As Icon = SystemImageListManager.GetIcon(index, True)

        Dim iconBounds As Rectangle
        Dim offset As Integer
        Dim displaytext As String = item.Text
        Try
            'Set the offset if the item is not in the Edit area
            If (e.State And DrawItemState.ComboBoxEdit) = DrawItemState.ComboBoxEdit Then
                offset = 1
                If m_displayPath And Not item.CShItem.Path.StartsWith("::") Then
                    displaytext = item.CShItem.Path
                End If
            Else
                offset = (item.Offset * m_offset) + 1
            End If

            iconBounds = New Rectangle(e.Bounds.Left + offset, e.Bounds.Top + _
                        CInt((Me.ItemHeight - ic.Height) / 2), ic.Width, ic.Height)

            If focus Then
                'Draw the background
                Dim textSize As SizeF = g.MeasureString(displaytext, e.Font)
                Dim focusRect As New Rectangle(iconBounds.Right + 2, iconBounds.Top, _
                                        CInt(textSize.Width) - 3, iconBounds.Height)
                g.FillRectangle(New SolidBrush(SystemColors.Highlight), focusRect)
            End If
            'If this is a hidden folder use ImageList_DrawEx to draw a transparent icon
            If item.CShItem.IsHidden Then
                Dim hDC As IntPtr = g.GetHdc
                If ImageList_DrawEx(SystemImageListManager.hSmallImageList, index, hDC, iconBounds.X, _
                      iconBounds.Y, 0, 0, -1, 0, ILD_TRANSPARENT Or ILD_SELECTED) = 0 Then
                    '0 = Not successful - just draw the DisplayName for the CShItem
                    g.DrawString(item.Text, e.Font, New SolidBrush(e.ForeColor), e.Bounds.Left, e.Bounds.Top)
                End If
                g.ReleaseHdc(hDC)
            Else
                If Not IsNothing(ic) Then
                    'Draw the Icon and the DisplayName for the CShItem
                    g.DrawString(displaytext, e.Font, New SolidBrush(e.ForeColor), _
                                iconBounds.Right, e.Bounds.Top + _
                                CInt((Me.ItemHeight - e.Font.Height) / 2))
                    g.DrawIcon(ic, iconBounds)
                    'Must use destroyIcon here as ic.dispose does not release the handle
                    DestroyIcon(ic.Handle)
                Else
                    'Draw the DisplayName for the CShItem
                    g.DrawString(displaytext, e.Font, New SolidBrush(e.ForeColor), e.Bounds.Left, e.Bounds.Top)
                End If
            End If
        Catch ex As Exception
            'If an exception occurs just draw the DisplayName
            If e.Index > -1 Then
                g.DrawString(displaytext, e.Font, New SolidBrush(e.ForeColor), e.Bounds.Left, e.Bounds.Top)
            End If
        End Try

    End Sub

#End Region

#Region " Overrides "

    'Make sure font doesn't get set to more than 12 points
    Protected Overrides Sub OnFontChanged(ByVal e As System.EventArgs)
        If Me.Font.SizeInPoints > 12 Then
            Me.Font = New Font(Me.Font.FontFamily, 12, Me.Font.Style)
        End If
    End Sub

    Protected Overrides Sub OnSelectedIndexChanged(ByVal e As System.EventArgs)
        If Me.SelectedIndex < 0 Then Exit Sub

        If m_playSound Then
            'Click
            Sounds.Click()
            m_playSound = False
        End If
        'Refresh the directories to allow for any changes
        Me.SelectedItem.CShItem.RefreshDirectories()
        MyBase.OnSelectedIndexChanged(e)

        'Check that this item is still valid
        Dim shItm As CShItem
        Dim parentItem As ExpComboItem = Me.SelectedItem.Parent
        If IsNothing(parentItem) Then GoTo OK

        Dim directories As ArrayList = parentItem.CShItem.GetDirectories
        For Each shItm In directories
            If shItm.Path = Me.SelectedItem.CShItem.Path Then GoTo OK
        Next
        'If we fall through here then the folder is no longer valid
        Me.Items.Remove(Me.SelectedItem)
        Me.SelectedItem = parentItem
        Exit Sub
OK:
        'Store the SelectedItem internally
        m_selectedItem = Me.SelectedItem

        'Check that all the children for this item are still valid
        Dim itm As ExpComboItem
        directories = Me.SelectedItem.CShItem.GetDirectories
        Dim invalidDirectories As New ArrayList
        For Each itm In Me.SelectedItem.Children
            If directories.IndexOf(itm.CShItem) < 0 Then
                invalidDirectories.Add(itm)
            End If
        Next

        'Remove any folders not in the currently selected path
        If m_style = ExpComboStyle.CurrentFolder Then
            For Each itm In Me.Items
                If Not itm.startUpItem And Not ExpCombo.IsAncestorOf(itm, m_selectedItem) Then
                    invalidDirectories.Add(itm)
                End If
            Next
        End If

        'Remove any invalid folders
        For Each itm In invalidDirectories
            If Not IsNothing(itm.Parent) Then itm.Parent.Children.Remove(itm)
            Me.Items.Remove(itm)
        Next

    End Sub

    Private Sub ExpCombo_DropDown(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.DropDown
        m_playSound = True
    End Sub

    Private Sub ExpCombo_Leave(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Leave
        m_playSound = False
    End Sub

#End Region

#Region " Public Methods "

    ''' <summary>
    ''' Builds the ExpCombo with the specified root item, adding and sorting all the subfolders of that item
    ''' </summary>
    ''' <param name="rootItem">The root item for the ExpCombo</param>
    ''' <returns>Boolean signifying success or failure</returns>
    ''' <remarks></remarks>
    Public Overridable Overloads Function BuildCombo(ByVal rootItem As CShItem) As Boolean
        Return BuildCombo(rootItem, True)
    End Function

    ''' <summary>
    ''' Builds the ExpCombo with the specified root item, adding all the subfolders of that item
    ''' </summary>
    ''' <param name="rootItem">The root item for the ExpCombo</param>
    ''' <param name="sort">Specifies whether the subfolders should be sorted</param>
    ''' <returns>Boolean signifying success or failure</returns>
    ''' <remarks></remarks>
    Public Overridable Overloads Function BuildCombo(ByVal rootItem As CShItem, ByVal sort As Boolean) As Boolean
        If Not rootItem.IsFolder Then Return False
        BuildCombo = False  'Assume failure
        Me.Items.Clear()
        If rootItem.Path = CShItem.GetDeskTop.Path Then
            'if the desktop is passed then just rebuild as normal
            'Fill ComboBox
            Dim itm As CShItem
            Dim i As Integer = 0

            Dim deskItem As ExpComboItem = Me.Items.AddRootItem(rootItem)
            deskItem.startUpItem = True
            Dim subItem As ExpComboItem

            'Get the directories and then sort them
            Dim dirList As ArrayList = rootItem.GetDirectories

            For Each itm In dirList
                subItem = New ExpComboItem(itm, deskItem)
                'Add item.
                i = Me.Items.Add(subItem, sort)
                subItem.startUpItem = True
                'Add all the items in MyComputer
                If itm.Path = CShItem.GetCShItem(ShellDll.CSIDL.DRIVES).Path Then
                    Dim shItem As CShItem
                    'Get the directories and then sort them
                    Dim drives As ArrayList = itm.GetDirectories
                    Dim myComp As ExpComboItem = subItem
                    For Each shItem In drives
                        subItem = New ExpComboItem(shItem, myComp)
                        Me.Items.Add(subItem, sort)
                        subItem.startUpItem = True
                    Next
                End If
            Next

        Else
            'Add the rootItem 
            Dim root As ExpComboItem = Me.Items.AddRootItem(rootItem)
            root.startUpItem = True

            'Add each of the subfolders
            Dim subItem As ExpComboItem
            Dim itm As CShItem
            'Get the directories and then sort them
            Dim dirList As ArrayList = rootItem.GetDirectories

            For Each itm In dirList
                subItem = New ExpComboItem(itm, root)
                Me.Items.Add(subItem, sort)
                subItem.startUpItem = True
            Next
        End If
        Return True
    End Function

    ''' <summary>
    ''' Selects a folder in the ExpCombo
    ''' </summary>
    ''' <param name="path">The path of the folder to be selected</param>
    ''' <returns>Boolean signifying success or failure</returns>
    ''' <remarks></remarks>
    Public Overloads Function SelectFolder(ByVal path As String) As Boolean
        Dim newItem As CShItem
        If path.ToLower.StartsWith("http://") Then
            'Remove trailing slash for SharePoint Folders
            If path.EndsWith("/") Then
                path = path.Substring(0, path.Length - 1)
            End If
        End If
        'This may return Nothing - but this will be caught 
        'in the SelectFolder(newItem As CShItem) Function
        If path.ToLower = CShItem.GetDeskTop.Path.ToLower Then
            newItem = CShItem.GetDeskTop
        Else
            newItem = CShItem.GetCShItem(path)
        End If
        Return SelectFolder(newItem)
    End Function

    ''' <summary>
    ''' Selects a folder in the ExpCombo
    ''' </summary>
    ''' <param name="item">The CShItem to be selected</param>
    ''' <returns>Boolean signifying success or failure</returns>
    ''' <remarks></remarks>
    Public Overloads Function SelectFolder(ByVal item As CShItem) As Boolean
        Me.BeginUpdate()

        'Exit if the nothing is passed
        If IsNothing(item) Then
            Me.EndUpdate()
            Exit Function
        End If

        If item.IsSharePoint Then
            Dim result As Boolean = SelectSharePointFolder(item)
            Me.EndUpdate()
            Return result
        End If

        'Exit if a non-folder is passed
        If Not item.IsFolder Then
            Me.EndUpdate()
            Exit Function
        End If

        'Set this Item as the root if the combo hasn't been initialized
        If IsNothing(Me.Items(0)) Then
            BuildCombo(item)
        End If

        Dim base As ExpComboItem = Me.Items(0)
        Dim testItem As CShItem
        Dim child As ExpComboItem

        'This is to make use of the MyDocuments folder that is directly under the Desktop
        'and the one directly under MyComputer.
        'This last folder only exists on XP systems where the computer is not in a domain.
        'I can't get a proper handle on it but the only time it will be used is if the Root
        'is MyComputer or the Desktop
        If item.Path = CShItem.GetCShItem(ShellDll.CSIDL.MYDOCUMENTS).Path And _
                   (base.CShItem.Path = CShItem.GetCShItem(ShellDll.CSIDL.DRIVES).Path Or _
                   base.CShItem.Path = CShItem.GetDeskTop.Path) Then
            'If MyComputer or the Desktop is the root then we must have MyDocuments somewhere in
            'the combo items - if not then we will just fall through and use the 
            'normal path under Documents and Settings
            For Each child In Me.Items
                If child.CShItem.Path = item.Path Then
                    Me.SelectedItem = child
                    Me.EndUpdate()
                    Return True
                End If
            Next
        End If
        Dim baseItem As CShItem = base.CShItem
        Dim index As Integer = 0
        'do the drill down 
        Dim cp As CShItem.cPidl = item.clsPidl      'the cPidl rep of the PIDL to be found
        Dim lim As Integer = cp.ItemCount - baseItem.clsPidl.ItemCount
        Do While lim > 0
            For Each testItem In baseItem.GetDirectories(True)
                If CShItem.IsAncestorOf(testItem, item, False) Then
                    For Each child In base.Children
                        If child.CShItem.Path = testItem.Path Then
                            index = Me.Items.IndexOf(child)
                            base = child
                            baseItem = testItem
                            lim -= 1
                            GoTo NEXTLEV
                        End If
                    Next
                    base = New ExpComboItem(testItem, base)
                    index = Me.Items.Add(base)
                    baseItem = testItem
                    lim -= 1
                    GoTo NEXTLEV
                End If
            Next
            'if fall thru then can't find it, return False
            Me.EndUpdate()
            Return False
NEXTLEV:
        Loop
        'after falling thru here, we have found the folder
        Me.SelectedIndex = index
        Me.EndUpdate()
        Return True
    End Function

    ''' <summary>
    ''' Selects a folder in the ExpCombo
    ''' </summary>
    ''' <param name="startDir">The member of the StartDir enumeration to be selected</param>
    ''' <returns>Boolean signifying success or failure</returns>
    ''' <remarks></remarks>
    Public Overloads Function SelectFolder(ByVal startDir As ExpTree.StartDir) As Boolean
        Dim special As CShItem
        If CType(Val(startDir), ExpTree.StartDir) = ExpTree.StartDir.Desktop Then
            special = CShItem.GetDeskTop()
        Else
            special = CShItem.GetCShItem(CType(Val(startDir), ShellDll.CSIDL))
        End If
        Return SelectFolder(special)
    End Function

    Private Function SelectSharePointFolder(ByVal itm As CShItem) As Boolean
        Dim network As CShItem = CShItem.GetCShItem(CSIDL.NETWORK)
        Dim networkItem As ExpComboItem

        If Not SelectFolder(network) Then Return False
        networkItem = Me.SelectedItem

        Dim parent As CShItem
        Dim child As CShItem
        Dim cboitm As ExpComboItem = Me.SelectedItem
        Dim itmList As New ArrayList
        Dim count As Integer

        For Each parent In network.GetDirectories
NEXTLEVEL:
            For Each child In parent.GetDirectories
                If Not child.IsSharePoint Then
                    Exit For
                ElseIf itm.Path.ToLower.StartsWith(child.Path.ToLower) Then
                    Debug.WriteLine(child.Path)
                    count = Me.Items.Count
                    cboitm = Me.Items(Me.Items.Add(New ExpComboItem(parent, cboitm)))
                    'If the number of items has changed, then store the item for removal on failure
                    If count < Me.Items.Count Then itmList.Add(cboitm)
                    Me.SelectedItem = cboitm
                    If child.Path.ToLower = itm.Path.ToLower Then
                        cboitm = Me.Items(Me.Items.Add(New ExpComboItem(child, cboitm)))
                        Me.SelectedItem = cboitm
                        Return True
                    End If
                    parent = child
                    GoTo NEXTLEVEL
                End If
            Next
            'The folder was not found here, remove any added Items
            For Each cboitm In itmList
                Me.Items.Remove(cboitm)
            Next
            itmList = New ArrayList
        Next
        Me.SelectedItem = networkItem
        Return Nothing
    End Function

    ''' <summary>
    ''' Refreshes the currently selected ExpComboItem
    ''' </summary>
    ''' <remarks></remarks>
    Public Overridable Sub RefreshCombo()
        If Not IsNothing(Me.SelectedItem) Then Me.SelectedItem.CShItem.RefreshDirectories()

        'Remove any folders not in the currently selected path
        Dim itm As ExpComboItem
        Dim invalidDirectories As New ArrayList
        If m_style = ExpComboStyle.CurrentFolder Then
            For Each itm In Me.Items
                If Not itm.startUpItem And Not ExpCombo.IsAncestorOf(itm, m_selectedItem) Then
                    invalidDirectories.Add(itm)
                End If
            Next
        End If

        'Remove any invalid folders
        For Each itm In invalidDirectories
            If Not IsNothing(itm.Parent) Then itm.Parent.Children.Remove(itm)
            Me.Items.Remove(itm)
        Next

    End Sub

    Public Shared Function IsAncestorOf(ByVal ancestor As ExpComboItem, ByVal current As ExpComboItem) As Boolean
        If IsNothing(ancestor) Or IsNothing(current) Then
            Return False
        End If
        If ancestor Is current Then Return True
        Dim parent As ExpComboItem = current.Parent
        Do While parent IsNot Nothing
            If ancestor Is parent Then Return True
            parent = parent.Parent
        Loop

    End Function

#End Region

End Class

''' <summary>
''' Enum for the style of an ExpCombo
''' </summary>
''' <remarks></remarks>
Public Enum ExpComboStyle
    ''' <summary>
    ''' Only shows the path to the currently selected folder in the list
    ''' </summary>
    ''' <remarks></remarks>
    CurrentFolder
    ''' <summary>
    ''' Leaves all folders in the list, even those not in the currently selected path
    ''' </summary>
    ''' <remarks></remarks>
    AllFolders
End Enum