Partial Class ExpCombo

    ''' <summary>
    ''' A collection class for the ExpCombo
    ''' </summary>
    ''' <remarks></remarks>
    Public Class ExpComboItemCollection
        Inherits ComboBox.ObjectCollection

        ''' <summary>
        ''' Creates a new collection
        ''' </summary>
        ''' <param name="owner">The parent ExpCombo</param>
        ''' <remarks></remarks>
        Public Sub New(ByVal owner As ExpCombo)
            MyBase.New(owner)
            m_expCombo = owner
        End Sub

        Private m_expCombo As ExpCombo
        ''' <summary>
        ''' The parent ExpCombo
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public ReadOnly Property ExpCombo() As ExpCombo
            Get
                Return m_expCombo
            End Get
        End Property

        ''' <summary>
        ''' Returns an ExpComboItem from the collection
        ''' </summary>
        ''' <param name="index">The index of the ExpComboItem to be returned</param>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Default Public Shadows ReadOnly Property Item(ByVal index As Integer) As ExpComboItem
            Get
                Return DirectCast(MyBase.Item(index), ExpComboItem)
            End Get
        End Property

        ''' <summary>
        ''' Adds an ExpComboItem to the collection
        ''' </summary>
        ''' <param name="item">The ExpComboItem to be added</param>
        ''' <returns>The index of the newly added ExpComboItem</returns>
        ''' <remarks>If an item with the same parent and the same path is found, the "item" parameter 
        ''' is set to this item and the existing items index is returned</remarks>
        Public Shadows Function Add(ByRef item As ExpComboItem) As Integer
            Return Add(item, True)
        End Function

        ''' <summary>
        ''' Adds an ExpComboItem to the collection
        ''' </summary>
        ''' <param name="item">The ExpComboItem to be added</param>
        ''' <param name="sort">Specifies whether the children of the parent item should be sorted</param>
        ''' <returns>The index of the newly added ExpComboItem</returns>
        ''' <remarks>If an item with the same parent and the same path is found, the "item" parameter 
        ''' is set to this item and the existing items index is returned</remarks>
        Public Shadows Function Add(ByRef item As ExpComboItem, ByVal sort As Boolean) As Integer

            'Make sure the item has been initialized
            If IsNothing(item.CShItem) Then
                Throw New ApplicationException("The CShItem property must be set before adding the item to the collection.")
            End If

            'Filter hidden folders if necessary
            If item.CShItem.IsHidden And Not m_expCombo.ShowHiddenFolders Then Exit Function

            'Set the DropDown width to the length of the item if it is too long
            'Add 16 pixels for the scrollbars and 16 pixels for the Icon - (item.offset + 2) * 16
            Dim length As Integer = CInt(m_expCombo.CreateGraphics.MeasureString(item.Text, m_expCombo.Font).Width + ((item.Offset + 2) * 16))
            If length + 5 > m_expCombo.DropDownWidth Then
                m_expCombo.DropDownWidth = length + 5  'Add another 5 pixel buffer just to keep up appearances
            End If

            'See if this is the root item
            If item.Parent Is Nothing Then
                'Clear all items then add the root item
                Clear()
                MyBase.Add(item)
                'This must be called to attach the collection to the ExpCombo. 
                m_expCombo.SetItemsCore(Me)
                Return IndexOf(item)
            End If

            'Check if the item has already been added
            Dim index As Integer = IndexOf(item)
            If index > -1 Then
                Return index
            End If
            'Add to the collection of children
            item.Parent.Children.Add(item)

            Dim parentIndex As Integer = Me.IndexOf(item.Parent)
            'Check that the parent exists in the list
            If parentIndex < 0 Then
                Throw New ApplicationException("Items parent has not been added to the collection.")
            End If

            'Sort the children
            If sort Then item.Parent.Children.Sort()

            'Get the new index of the child in the parents Children array
            Dim childIndex As Integer = item.Parent.Children.IndexOf(item)
            Dim sibling As ExpComboItem
            Select Case childIndex
                'if the child is not the last in the list then 
                'we can insert it just before the next child
                'otherwise we have to find the next sibling of
                'the parent folder and insert just before that.
                Case Is < item.Parent.Children.Count - 1
                    sibling = DirectCast(item.Parent.Children.Item(childIndex + 1), ExpComboItem)
                    'Insert item directly before the next sibling
                    index = MyBase.IndexOf(sibling)
                    MyBase.Insert(index, item)
                    'This must be called to attach the collection to the ExpCombo. 
                    m_expCombo.SetItemsCore(Me)
                    Return index
                Case Else
                    index = parentIndex
                    Do
                        'increment the index
                        index += 1
                        'if the index is out of the range of the 
                        'ComboBox items then we are at the end
                        'of the list and we can just add the item 
                        'to the end of the list.
                        If index >= Me.Count Then
                            MyBase.Add(item)
                            'This must be called to attach the collection to the ExpCombo. 
                            m_expCombo.SetItemsCore(Me)
                            Return IndexOf(item)
                        End If
                        'Check the next folder to see if it has the same (or less)
                        'offset level as the parent. If so the insert the item directly before it.
                        sibling = Me.Item(index)
                        If sibling.Offset <= item.Parent.Offset Then
                            MyBase.Insert(index, item)
                            'This must be called to attach the collection to the ExpCombo. 
                            m_expCombo.SetItemsCore(Me)
                            Return index
                        End If
                    Loop
            End Select
        End Function

        ''' <summary>
        ''' Adds an array of ExpComboItems to the collection
        ''' </summary>
        ''' <param name="items">The array of ExpComboItems to be added</param>
        ''' <remarks></remarks>
        Public Shadows Sub AddRange(ByVal items As ExpComboItem())
            Dim itm As ExpComboItem
            For Each itm In items
                Add(itm)
            Next
        End Sub

        ''' <summary>
        ''' Inserts an ExpComboItem into the collection
        ''' </summary>
        ''' <param name="index">The index at which to insert the ExpComboItem</param>
        ''' <param name="item">The ExpComboItem to insert</param>
        ''' <remarks></remarks>
        Public Shadows Sub Insert(ByVal index As Integer, ByVal item As ExpComboItem)

            'Make sure the item has been initialized
            If IsNothing(item.CShItem) Then
                Throw New ApplicationException("The CShItem property must be set before adding the item to the collection.")
            End If

            'Check if the index is in the range allowed for the parent item
            Dim parentIndex As Integer = Me.IndexOf(item.Parent)
            'Check that the parent exists in the list
            If parentIndex < 0 Then
                Throw New ApplicationException("Items parent has not been added to the collection.")
            End If

            If index <= parentIndex Or _
                index > parentIndex + item.Parent.Children.Count + 1 Then
                Throw New ApplicationException("Index does not fall within the acceptable " & _
                                                "range for the items parent.")
            Else
                'Check if the item has already been added
                Dim testIndex As Integer = IndexOf(item)
                If testIndex > -1 Then
                    Throw New ApplicationException("Item already exists in the collection.")
                End If

                'Add the item
                item.Parent.Children.Add(item)
                MyBase.Insert(index, item)
                'This must be called to attach the collection to the ExpCombo. 
                m_expCombo.SetItemsCore(Me)

            End If
        End Sub

        ''' <summary>
        ''' Clears the collection and adds a new root item
        ''' </summary>
        ''' <param name="rootItem">The ExpComboItem to be added as the root of the collection</param>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public Function AddRootItem(ByVal rootItem As CShItem) As ExpComboItem
            'Clear the list
            Me.Clear()
            'Add the new item without a parent
            Dim root As New ExpComboItem(rootItem, Nothing)
            Add(root)
            Return root
        End Function

        ''' <summary>
        ''' Returns the index of the specified ExpComboItem
        ''' </summary>
        ''' <param name="item">The ExpComboItem to retrieve the index for</param>
        ''' <returns>The index of the specified ExpComboItem</returns>
        ''' <remarks>If an item with the same parent and the same path is found, the "item" parameter 
        ''' is set to this item and the existing items index is returned</remarks>
        Public Shadows Function IndexOf(ByRef item As ExpComboItem) As Integer
            Dim comboItem As ExpComboItem
            If Not IsNothing(item.Parent) Then
                For Each comboItem In item.Parent.Children
                    If comboItem.CShItem.Path = item.CShItem.Path Then
                        item = comboItem
                        Return MyBase.IndexOf(comboItem)
                    End If
                Next
            End If
            Return MyBase.IndexOf(item)
        End Function

        ''' <summary>
        ''' Removes an ExpComboItem from the collection
        ''' </summary>
        ''' <param name="item">The ExpComboItem to be removed</param>
        Public Shadows Sub Remove(ByVal item As ExpComboItem)
            Dim itm As ExpComboItem = m_expCombo.SelectedItem
            MyBase.Remove(item)
            'This must be called to attach the collection to the ExpCombo. 
            m_expCombo.SetItemsCore(Me)
            If Not itm Is item Then
                m_expCombo.SelectedItem = itm
            End If
        End Sub

        ''' <summary>
        ''' Removes an ExpComboItem from the collection
        ''' </summary>
        ''' <param name="index">The index of the ExpComboItem to be removed</param>
        Public Shadows Sub RemoveAt(ByVal index As Integer)
            Dim itm As ExpComboItem = Me.Item(index)
            Remove(itm)
        End Sub

    End Class

End Class
