Option Strict On

Imports System.Resources
Imports System.Threading
Imports System.Globalization
Imports System.ComponentModel
Imports System.Runtime.InteropServices

''' <summary>
''' ColumnHeader Class for the ExpList Control
''' </summary>
''' <remarks></remarks>
<DefaultEvent("ColumnAdded"), ClassInterface(ClassInterfaceType.None), _
Guid("46D8DD31-3972-4370-97DB-80C4D262CE66")> _
Public Class ExpListColumn
    Inherits ColumnHeader

#Region " Events "

    ''' <summary>
    ''' Event fired when the column is added to the parent ExpLists column collection
    ''' </summary>
    ''' <remarks></remarks>
    Public Event ColumnAdded As EventHandler
    ''' <summary>
    ''' Event fired when the column is removed from the parent ExpLists column collection
    ''' </summary>
    ''' <remarks></remarks>
    Public Event ColumnRemoved As EventHandler

#End Region

#Region " Fields and Properties "
    ''' <summary>
    ''' The MenuItem in the "Arrange Icons By" flyout menu corresponding to this column
    ''' </summary>
    ''' <remarks></remarks>
    Friend WithEvents sortMenuItem As ToolStripMenuItem
    ''' <summary>
    ''' MenuItem shown in the ColumnContextMenu of the parent ExpList
    ''' </summary>
    ''' <remarks></remarks>
    Friend WithEvents contextMenuItem As ToolStripMenuItem

    Private m_columnType As ExpListColumnType
    ''' <summary>
    ''' The Type of the column
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks>This can be any of the values of the ExpListColumnType Enumeration</remarks>
    <Category("Behavior"), Description("Sets the file information that will be displayed in this column"), _
    DefaultValue(GetType(ExpListColumnType), "None")> _
    Public Property ColumnType() As ExpListColumnType
        Get
            Return m_columnType
        End Get
        Set(ByVal value As ExpListColumnType)
            m_columnType = value
            'Set up the column again
            Initialize()
            'Check that we have initialized the m_expList variable
            'this is required for designer support
            If IsNothing(m_expList) Then
                'Exit if the base listview hasn't been initialized
                If IsNothing(MyBase.ListView) Then Exit Property
                m_expList = DirectCast(MyBase.ListView, ExpList)
            End If

            If Me.Index < 0 Then Exit Property
            'Reset the text for this column
            Dim item As ExpListItem
            For Each item In m_expList.Items
                m_expList.SetItemText(item, Me)
            Next
        End Set
    End Property

    Private m_customGrouping As Boolean = False
    ''' <summary>
    ''' Sets whether custom grouping is enabled. If so the caller must set the Groups manually.
    ''' </summary>
    ''' <remarks></remarks>
    <Browsable(False), DefaultValue(False)> _
    Public Property CustomGrouping() As Boolean
        Get
            Return m_customGrouping
        End Get
        Set(ByVal Value As Boolean)
            m_customGrouping = Value
        End Set
    End Property

    Private m_expList As ExpList
    ''' <summary>
    ''' The parent ExpList Object
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Browsable(False)> _
    Public Property ExpList() As ExpList
        Get
            Return m_expList
        End Get
        Set(ByVal value As ExpList)
            m_expList = value
        End Set
    End Property

    Private m_sorter As IComparer
    ''' <summary>
    ''' The IComparer to be used for sorting this column
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks>The default is the ExpListComparer Class. This will sort 
    ''' most custom column types correctly</remarks>
    <Browsable(False), DefaultValue(CType(Nothing, Object))> _
    Public Property ExpListItemSorter() As IComparer
        Get
            Return m_sorter
        End Get
        Set(ByVal value As IComparer)
            m_sorter = value
        End Set
    End Property

    ''' <summary>
    ''' Hide the inherited ListView Property to force the user to use the ExpList Property
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Shadows ReadOnly Property ListView() As ExpList
        Get
            Return m_expList
        End Get
    End Property

    ''' <summary>
    ''' The Text shown on the ColumnHeader and the ContextMenu Items
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Appearance"), Description("The Text shown on the ColumnHeader and the ContextMenu Items")> _
    Public Shadows Property Text() As String
        Get
            Return MyBase.Text
        End Get
        Set(ByVal value As String)
            MyBase.Text = value
            contextMenuItem.Text = value
            sortMenuItem.Text = value
        End Set
    End Property

    Private m_visible As Boolean = True
    ''' <summary>
    ''' Gets or sets whether the column is currently visible in the parent ExpList
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Behavior"), Description("Gets or sets whether the column is currently visible in " & _
    "the parent ExpList (all columns are always visible in design mode)"), DefaultValue(True)> _
    Public Overridable Property Visible() As Boolean
        Get
            Return m_visible
        End Get
        Set(ByVal value As Boolean)
            m_visible = value
            If Not Me.DesignMode Then
                ShowColumn(value)
            End If
        End Set
    End Property

#End Region

#Region " Constructors "

    ''' <summary>
    '''  Constructor for designer support
    ''' </summary>
    ''' <remarks></remarks>
    <EditorBrowsable(EditorBrowsableState.Never), Browsable(False)> _
    Public Sub New()
        MyBase.New()
        '(initialization calls are made from the ColumnType Property)
        ColumnType = ExpListColumnType.None
    End Sub

    ''' <summary>
    '''  Constructor using the default type (ExpListColumnType.None)
    ''' </summary>
    ''' <param name="parent">The parent ExpList</param>
    ''' <remarks></remarks>
    Public Sub New(ByVal parent As ExpList)
        Me.New(parent, ExpListColumnType.None)
    End Sub

    ''' <summary>
    ''' Constructor using the specified type 
    ''' </summary>
    ''' <param name="parent">The parent ExpList</param>
    ''' <param name="type">The column type</param>
    ''' <remarks></remarks>
    Public Sub New(ByVal parent As ExpList, ByVal type As ExpListColumnType)

        MyBase.New()

        'Store the column type and the parent
        '(initialization calls are made from the ColumnType Property)
        m_expList = parent
        ColumnType = type

    End Sub
    ''TODO: parent???
    ''' <summary>
    '''  Constructor using the default type (ExpListColumnType.None)
    ''' </summary>
    ''' <param name="parent">The parent Browser</param>
    ''' <remarks></remarks>
    Public Sub New(ByVal parent As Browser)
        Me.New(parent, ExpListColumnType.None)
    End Sub

    ''' <summary>
    ''' Constructor using the specified type 
    ''' </summary>
    ''' <param name="parent">The parent Browser</param>
    ''' <param name="type">The column type</param>
    ''' <remarks></remarks>
    Public Sub New(ByVal parent As Browser, ByVal type As ExpListColumnType)

        MyBase.New()

        'Store the column type and the parent
        '(initialization calls are made from the ColumnType Property)
        m_expList = parent.expLV
        ColumnType = type

    End Sub

    Private Sub Initialize()

        'Initialize the menu items if necessary
        If contextMenuItem Is Nothing Then
            contextMenuItem = New ToolStripMenuItem
            contextMenuItem.Tag = Me
        End If
        If sortMenuItem Is Nothing Then sortMenuItem = New ToolStripMenuItem

        'Set up the column 
        Select Case m_columnType
            Case ExpListColumnType.Name
                Me.Text = My.Resources.ControlStrings.Name
                Me.Width = 130
            Case ExpListColumnType.Path
                Me.Text = My.Resources.ControlStrings.Path
                Me.Width = 185
            Case ExpListColumnType.Type
                Me.Text = My.Resources.ControlStrings.Type
                Me.Width = 90
            Case ExpListColumnType.Size
                Me.Text = My.Resources.ControlStrings.Size
                Me.Width = 50
                Me.TextAlign = HorizontalAlignment.Right
            Case ExpListColumnType.Modified
                Me.Text = My.Resources.ControlStrings.Modified
                sortMenuItem.Text = My.Resources.ControlStrings.ModifiedContext
                Me.Width = 85
            Case ExpListColumnType.Created
                Me.Text = My.Resources.ControlStrings.Created
                sortMenuItem.Text = My.Resources.ControlStrings.CreatedContext
                Me.Width = 85
            Case ExpListColumnType.CustomDate, ExpListColumnType.CustomNumeric, ExpListColumnType.CustomText
                'the user is responsible for setting the text for the column
                Me.Text = ""
                Me.Width = 130
        End Select

    End Sub

    Friend Sub SetText()
        Select Case m_columnType
            Case ExpListColumnType.Name
                Me.Text = My.Resources.ControlStrings.Name
            Case ExpListColumnType.Path
                Me.Text = My.Resources.ControlStrings.Path
            Case ExpListColumnType.Type
                Me.Text = My.Resources.ControlStrings.Type
            Case ExpListColumnType.Size
                Me.Text = My.Resources.ControlStrings.Size
            Case ExpListColumnType.Modified
                Me.Text = My.Resources.ControlStrings.Modified
                sortMenuItem.Text = My.Resources.ControlStrings.ModifiedContext
            Case ExpListColumnType.Created
                Me.Text = My.Resources.ControlStrings.Created
                sortMenuItem.Text = My.Resources.ControlStrings.CreatedContext
        End Select
    End Sub

#End Region

#Region " Methods "

#Region "    Control Event Handling "

    Private Sub m_contextMenuItem_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles contextMenuItem.Click
        Me.Visible = Not contextMenuItem.Checked
        'Click
        Sounds.Click()
    End Sub

    Private Sub m_sortMenuItem_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles sortMenuItem.Click
        'Figure out the new sorting order and call SortByColumn.
        If (Me Is m_expList.SortColumn) And (m_expList.SortOrder = SortOrder.Ascending) Then
            m_expList.SortByColumn(Me, SortOrder.Descending)
        Else
            m_expList.SortByColumn(Me, SortOrder.Ascending)
        End If

        'Click
        Sounds.Click()
    End Sub

#End Region

#Region "    Protected Event Raising Methods "

    ''' <summary>
    ''' Raises the ColumnAdded Event when the column is added to the parent ExpLists column collection
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Protected Overridable Sub OnColumnAdded(ByVal e As EventArgs)
        RaiseEvent ColumnAdded(Me, e)
    End Sub

    ''' <summary>
    ''' Raises the ColumnRemoved Event when the column is removed from the parent ExpLists column collection
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Protected Overridable Sub OnColumnRemoved(ByVal e As EventArgs)
        RaiseEvent ColumnRemoved(Me, e)
    End Sub

#End Region

    Private Sub ShowColumn(ByVal show As Boolean)

        'Check that we have initialized the m_expList variable
        'this is required for designer support
        If IsNothing(m_expList) Then
            'Exit if the base listview hasn't been initialized
            If IsNothing(MyBase.ListView) Then Exit Sub
            m_expList = DirectCast(MyBase.ListView, ExpList)
        End If

        If show Then
            'Add the column 
            m_expList.Columns.Add(Me)
            'Raise an event so that any custom columns
            'can have their text updated 
            OnColumnAdded(New EventArgs)
        Else
            'Check that the column exists in the collection
            If m_expList.Columns.IndexOf(Me) < 0 Then Exit Sub

            'Remove the column
            m_expList.Columns.Remove(Me)

            'Raise the ColumnRemoved Event
            OnColumnRemoved(New EventArgs)
        End If
    End Sub

#End Region

End Class

